;; Copyright (C) 2023  Felisp
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU Affero General Public License as published by
;; the Free Software Foundation, version 3 of the License.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU Affero General Public License for more details.
;;
;; You should have received a copy of the GNU Affero General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>.

(ns rss-thread-watch.utils
  "Util functions"
  (:gen-class))

;; ===== Macros =====
(defmacro nil?-else
  "Return x unless it's nil, the return y"
  [x y]
  `(let [result# ~x]
        (if (nil? result#)
          ~y
          result#)))

(defmacro when-else
  "Evaluates [tst], if it's truthy value returns that value.
   If it's not, execute everything in [else] and return last expr."
  [tst & else]
  `(let [res# ~tst]
     (if res#
       res#
       (do ~@else))))

(defmacro ret=
  "compares two values using [=]. If the result is true
   returns the value, else the result of [=].

   Usefull with if-else"
  [x y]
  `(let [x# ~x
         y# ~y
         result# ~(= x y)]
     (if result#
       ~x
       result#)))

;; ===== Generic functions ====

(defn indices
  ;; https://stackoverflow.com/questions/8641305/find-index-of-an-element-matching-a-predicate-in-clojure
  "Returns indexes of elements passing predicate"
  [pred coll]
   (keep-indexed #(when (pred %2) %1) coll))

(defn map-apply-defaults
  "Apply default values from [defaults] to keys not present in [conf]
   Order is very important.
   Thus all missing values from config are replaced by defaults"
  [conf defaults]
  (into conf
        (for [k (keys defaults)]
          (let [val1 (get conf k)
                val2 (get defaults k)]
            (if (and (map? val1)
                     (map? val2))
              {k (map-deep-merge-missing val1 val2)}
              {k (nil?-else val2 val1)})))))
